@@ -1,25 +1,20 @@
 package Bitcoin::Crypto::Key::ExtPrivate;
 
-our $VERSION = "0.997_01";
-
-use v5.10;
-use warnings;
+use Modern::Perl "2010";
 use Moo;
-use Crypt::Mac::HMAC qw(hmac);
+use Digest::SHA qw(hmac_sha512);
+use Math::BigInt 1.999816 try => 'GMP';
+use Math::EllipticCurve::Prime;
 use Encode qw(encode decode);
 use Unicode::Normalize;
-use Bitcoin::BIP39 qw(gen_bip39_mnemonic bip39_mnemonic_to_entropy entropy_to_bip39_mnemonic);
-use Crypt::KeyDerivation qw(pbkdf2);
-use Scalar::Util qw(blessed);
+use Bitcoin::BIP39 qw(gen_bip39_mnemonic bip39_mnemonic_to_entropy);
+use PBKDF2::Tiny qw(derive);
 
-use Bitcoin::Crypto::BIP44;
 use Bitcoin::Crypto::Key::ExtPublic;
 use Bitcoin::Crypto::Config;
-use Bitcoin::Crypto::Helpers qw(new_bigint pad_hex ensure_length verify_bytestring);
+use Bitcoin::Crypto::Helpers qw(pad_hex ensure_length);
 use Bitcoin::Crypto::Exception;
 
-use namespace::clean;
-
 with "Bitcoin::Crypto::Role::ExtendedKey";
 
 sub _is_private { 1 }
@@ -36,27 +31,8 @@ sub generate_mnemonic
 		"required entropy of between $min_len and $max_len bits, divisible by $len_div"
 	) if $len < $min_len || $len > $max_len || $len % $len_div != 0;
 
-	return Bitcoin::Crypto::Exception::MnemonicGenerate->trap_into(
-		sub {
-			my $ret = gen_bip39_mnemonic(bits => $len, language => $lang);
-			$ret->{mnemonic};
-		}
-	);
-}
-
-sub mnemonic_from_entropy
-{
-	my ($class, $entropy, $lang) = @_;
-	$lang //= "en";
-
-	return Bitcoin::Crypto::Exception::MnemonicGenerate->trap_into(
-		sub {
-			entropy_to_bip39_mnemonic(
-				entropy => $entropy,
-				language => $lang
-			);
-		}
-	);
+	my $ret = gen_bip39_mnemonic(bits => $len, language => $lang);
+	return $ret->{mnemonic};
 }
 
 sub from_mnemonic
@@ -66,19 +42,11 @@ sub from_mnemonic
 	$password = encode("UTF-8", NFKD(decode("UTF-8", "mnemonic" . ($password // ""))));
 
 	if (defined $lang) {
-
 		# checks validity of seed in given language
 		# requires Wordlist::LANG::BIP39 module for given LANG
-		Bitcoin::Crypto::Exception::MnemonicCheck->trap_into(
-			sub {
-				bip39_mnemonic_to_entropy(
-					mnemonic => $mnemonic,
-					language => $lang
-				);
-			}
-		);
+		bip39_mnemonic_to_entropy(mnemonic => $mnemonic, language => $lang);
 	}
-	my $bytes = pbkdf2($mnemonic, $password, 2048, "SHA512", 64);
+	my $bytes = derive("SHA-512", $mnemonic, $password, 2048);
 
 	return $class->from_seed($bytes);
 }
@@ -86,9 +54,7 @@ sub from_mnemonic
 sub from_seed
 {
 	my ($class, $seed) = @_;
-	verify_bytestring($seed);
-
-	my $bytes = hmac("SHA512", "Bitcoin seed", $seed);
+	my $bytes = hmac_sha512($seed, "Bitcoin seed");
 	my $key = substr $bytes, 0, 32;
 	my $cc = substr $bytes, 32, 32;
 
@@ -118,44 +84,29 @@ sub get_public_key
 	return $public;
 }
 
-sub bip44_derive_key
-{
-	my ($self, %data) = @_;
-	my $path = Bitcoin::Crypto::BIP44->new(
-		%data,
-		coin_type => $self,
-	);
-
-	return $self->derive_key($path);
-}
-
 sub _derive_key_partial
 {
 	my ($self, $child_num, $hardened) = @_;
 
 	my $hmac_data;
 	if ($hardened) {
-
 		# zero byte
 		$hmac_data .= "\x00";
-
 		# key data - 32 bytes
 		$hmac_data .= ensure_length $self->raw_key, $config{key_max_length};
-	}
-	else {
+	} else {
 		# public key data - SEC compressed form
 		$hmac_data .= $self->raw_key("public_compressed");
 	}
-
 	# child number - 4 bytes
 	$hmac_data .= ensure_length pack("N", $child_num), 4;
 
-	my $data = hmac("SHA512", $self->chain_code, $hmac_data);
+	my $data = hmac_sha512($hmac_data, $self->chain_code);
 	my $chain_code = substr $data, 32, 32;
 
-	my $number = new_bigint(substr $data, 0, 32);
-	my $key_num = new_bigint($self->raw_key);
-	my $n_order = new_bigint(pack "H*", $self->key_instance->curve2hash->{order});
+	my $number = Math::BigInt->from_bytes(substr $data, 0, 32);
+	my $key_num = Math::BigInt->from_bytes($self->raw_key);
+	my $n_order = Math::EllipticCurve::Prime->from_name($config{curve_name})->n;
 
 	Bitcoin::Crypto::Exception::KeyDerive->raise(
 		"key $child_num in sequence was found invalid"
@@ -164,11 +115,7 @@ sub _derive_key_partial
 	$number->badd($key_num);
 	$number->bmod($n_order);
 
-	Bitcoin::Crypto::Exception::KeyDerive->raise(
-		"key $child_num in sequence was found invalid"
-	) if $number->beq(0);
-
-	return (blessed $self)->new(
+	return __PACKAGE__->new(
 		$number->as_bytes,
 		$chain_code,
 		$child_num,
@@ -182,7 +129,7 @@ sub _derive_key_partial
 __END__
 =head1 NAME
 
-Bitcoin::Crypto::Key::ExtPrivate - Bitcoin extended private keys
+Bitcoin::Crypto::Key::ExtPrivate - class for Bitcoin extended private keys
 
 =head1 SYNOPSIS
 
@@ -239,20 +186,6 @@ Other languages than english require additional modules for L<Bitcoin::BIP39>.
 Dies when $len is invalid (under 128, above 256 or not divisible by 32).
 Returns newly generated BIP39 mnemonic string.
 
-In some environments a problem may be encountered that causes the secure random bytes generator to block the program execution (See L<Bytes::Random::Secure/"BLOCKING ENTROPY SOURCE">). In this case you can use I<mnemonic_from_entropy> and pass in entropy generated by L<Bytes::Random::Secure> in non-blocking mode (via the OO interface).
-
-=head2 mnemonic_from_entropy
-
-	sig: mnemonic_from_entropy($class, $bytes, $lang = "en")
-
-Generates a new mnemonic code from custom entropy given in $bytes (a bytestring). This entropy should be of the same bit size as in L</"generate_mnemonic">. Returns newly generated BIP39 mnemonic string.
-
-This can be useful to avoid relying on the underlying implementation of L<Bitcoin::BIP39>.
-
-Another use would be implementing one's own entropy source that can be truly random, not just cryptographically-secure. A popular example would be capturing user's mouse movements.
-
-Be aware that the method you use to generate a mnemonic will be a very important factor in your key's security. If possible, use real sources of randomness (not pseudo-random) or a cryptographically secure pseduo-random number generator like the one used by L<Bytes::Random::Secure>.
-
 =head2 from_mnemonic
 
 	sig: from_mnemonic($class, $mnemonic, $password = "", $lang = undef)
@@ -301,7 +234,7 @@ Dies on errors. If multiple networks match serialized data specify $network manu
 
 =head2 from_serialized_base58
 
-	sig: from_serialized_base58($class, $base58, $network = undef)
+	sig: from_serialized_base58($class, $base58, $network)
 
 Same as from_serialized, but performs Base58Check decoding on $base58 argument.
 
@@ -309,7 +242,7 @@ Same as from_serialized, but performs Base58Check decoding on $base58 argument.
 
 	sig: set_network($self, $val)
 
-Change key's network state to $val. It can be either network name present in Bitcoin::Crypto::Network package or an instance of this class.
+Change key's network state to $val. It can be either network name present in Bitcoin::Crypto::Network package or a valid network hashref.
 
 Returns current key instance.
 
@@ -329,19 +262,11 @@ Returns the key in basic format: L<Bitcoin::Crypto::Key::Private>
 
 	sig: derive_key($self, $path)
 
-Performs extended key derivation as specified in BIP32 on the current key with $path. Dies on error.
+Performs extended key deriviation as specified in BIP32 on the current key with $path. Dies on error.
 
-See BIP32 document for details on derivation paths and methods.
+See BIP32 document for details on deriviation paths and methods.
 
-Returns a new extended key instance - result of a derivation.
-
-=head2 bip44_derive_key
-
-	sig: bip44_derive_key($self, %data)
-
-A helper that constructs a L<Bitcoin::Crypto::BIP44> path from C<%data> and calls L</derive_key> with it. Refer to L<Bitcoin::Crypto::BIP44/PROPERTIES> to see what you can include in C<%data>.
-
-I<Note: coin_type parameter will be ignored, and the current network configuration set in the extended key will be used.>
+Returns a new extended key instance - result of a deriviation.
 
 =head2 get_fingerprint
 
@@ -355,15 +280,13 @@ This module throws an instance of L<Bitcoin::Crypto::Exception> if it encounters
 
 =over 2
 
-=item * MnemonicGenerate - mnemonic couldn't be generated correctly
+=item MnemonicGenerate - mnemonic couldn't be generated correctly
 
-=item * MnemonicCheck - mnemonic didn't pass the validity check
+=item KeyDerive - key couldn't be derived correctly
 
-=item * KeyDerive - key couldn't be derived correctly
+=item KeyCreate - key couldn't be created correctly
 
-=item * KeyCreate - key couldn't be created correctly
-
-=item * NetworkConfig - incomplete or corrupted network configuration
+=item NetworkConfig - incomplete or corrupted network configuration
 
 =back
 
