@@ -1,19 +1,16 @@
 package Bitcoin::Crypto::Key::ExtPublic;
 
-our $VERSION = "0.997_01";
-
-use v5.10;
-use warnings;
+use Modern::Perl "2010";
 use Moo;
-use Crypt::Mac::HMAC qw(hmac);
-use Scalar::Util qw(blessed);
+use Digest::SHA qw(hmac_sha512);
+use Math::BigInt 1.999816 try => 'GMP';
+use Math::EllipticCurve::Prime;
+use Math::EllipticCurve::Prime::Point;
 
 use Bitcoin::Crypto::Config;
-use Bitcoin::Crypto::Helpers qw(new_bigint ensure_length add_ec_points);
+use Bitcoin::Crypto::Helpers qw(ensure_length);
 use Bitcoin::Crypto::Exception;
 
-use namespace::clean;
-
 with "Bitcoin::Crypto::Role::ExtendedKey";
 
 sub _is_private { 0 }
@@ -28,30 +25,30 @@ sub _derive_key_partial
 
 	# public key data - SEC compressed form
 	my $hmac_data = $self->raw_key("public_compressed");
-
 	# child number - 4 bytes
 	$hmac_data .= ensure_length pack("N", $child_num), 4;
 
-	my $data = hmac("SHA512", $self->chain_code, $hmac_data);
+	my $data = hmac_sha512($hmac_data, $self->chain_code);
 	my $chain_code = substr $data, 32, 32;
 
-	my $n_order = new_bigint(pack "H*", $self->key_instance->curve2hash->{order});
-	my $number = new_bigint(substr $data, 0, 32);
+	my $el_curve = Math::EllipticCurve::Prime->from_name($config{curve_name});
+	my $number = Math::BigInt->from_bytes(substr $data, 0, 32);
+	my $key = $self->_create_key(substr $data, 0, 32);
+	my $point = Math::EllipticCurve::Prime::Point->from_bytes($key->export_key_raw("public"));
+	$point->curve($el_curve);
+	my $point_cpy = $point->copy();
+	my $parent_point = Math::EllipticCurve::Prime::Point->from_bytes($self->raw_key("public"));
+	$parent_point->curve($el_curve);
+	my $n_order = $el_curve->n;
+
+	$point->badd($parent_point);
+
 	Bitcoin::Crypto::Exception::KeyDerive->raise(
 		"key $child_num in sequence was found invalid"
 	) if $number->bge($n_order);
 
-	my $key = $self->_create_key(substr $data, 0, 32);
-	my $point = $key->export_key_raw("public");
-	my $parent_point = $self->raw_key("public");
-	$point = add_ec_points($point, $parent_point);
-
-	Bitcoin::Crypto::Exception::KeyDerive->raise(
-		"key $child_num in sequence was found invalid"
-	) unless defined $point;
-
-	return (blessed $self)->new(
-		$point,
+	return __PACKAGE__->new(
+		$point->to_bytes,
 		$chain_code,
 		$child_num,
 		$self->get_fingerprint,
@@ -64,7 +61,7 @@ sub _derive_key_partial
 __END__
 =head1 NAME
 
-Bitcoin::Crypto::Key::ExtPublic - Bitcoin extended public keys
+Bitcoin::Crypto::Key::ExtPublic - class for Bitcoin extended public keys
 
 =head1 SYNOPSIS
 
@@ -122,7 +119,7 @@ Dies on errors. If multiple networks match serialized data specify $network manu
 
 =head2 from_serialized_base58
 
-	sig: from_serialized_base58($class, $base58, $network = undef)
+	sig: from_serialized_base58($class, $base58, $network)
 
 Same as from_serialized, but performs Base58Check decoding on $base58 argument.
 
@@ -130,7 +127,7 @@ Same as from_serialized, but performs Base58Check decoding on $base58 argument.
 
 	sig: set_network($self, $val)
 
-Change key's network state to $val. It can be either network name present in Bitcoin::Crypto::Network package or an instance of this class.
+Change key's network state to $val. It can be either network name present in Bitcoin::Crypto::Network package or a valid network hashref.
 
 Returns current key instance.
 
@@ -144,13 +141,13 @@ Returns the key in basic format: L<Bitcoin::Crypto::Key::Public>
 
 	sig: derive_key($self, $path)
 
-Performs extended key derivation as specified in BIP32 on the current key with $path. Dies on error.
+Performs extended key deriviation as specified in BIP32 on the current key with $path. Dies on error.
 
-See BIP32 document for details on derivation paths and methods.
+See BIP32 document for details on deriviation paths and methods.
 
-Note that public keys cannot derive private keys and your derivation path must start with M (capital m).
+Note that public keys cannot derive private keys and your deriviation path must start with M (capital m).
 
-Returns a new extended key instance - result of a derivation.
+Returns a new extended key instance - result of a deriviation.
 
 =head2 get_fingerprint
 
@@ -164,11 +161,11 @@ This module throws an instance of L<Bitcoin::Crypto::Exception> if it encounters
 
 =over 2
 
-=item * KeyDerive - key couldn't be derived correctly
+=item KeyDerive - key couldn't be derived correctly
 
-=item * KeyCreate - key couldn't be created correctly
+=item KeyCreate - key couldn't be created correctly
 
-=item * NetworkConfig - incomplete or corrupted network configuration
+=item NetworkConfig - incomplete or corrupted network configuration
 
 =back
 
