@@ -1,11 +1,8 @@
 package Bitcoin::Crypto::Key::Private;
 
-our $VERSION = "0.997_01";
-
-use v5.10;
-use warnings;
+use Modern::Perl "2010";
 use Moo;
-use Types::Standard qw(Str);
+use MooX::Types::MooseLike::Base qw(Str);
 use Crypt::PK::ECC;
 use Bitcoin::BIP39 qw(bip39_mnemonic_to_entropy entropy_to_bip39_mnemonic);
 use List::Util qw(first);
@@ -13,13 +10,11 @@ use List::Util qw(first);
 use Bitcoin::Crypto::Key::Public;
 use Bitcoin::Crypto::Base58 qw(encode_base58check decode_base58check);
 use Bitcoin::Crypto::Config;
-use Bitcoin::Crypto::Network;
+use Bitcoin::Crypto::Network qw(find_network get_network);
 use Bitcoin::Crypto::Util qw(validate_wif);
 use Bitcoin::Crypto::Helpers qw(ensure_length);
 use Bitcoin::Crypto::Exception;
 
-use namespace::clean;
-
 with "Bitcoin::Crypto::Role::BasicKey";
 
 sub _is_private { 1 }
@@ -28,13 +23,10 @@ sub to_wif
 {
 	my ($self) = @_;
 	my $bytes = $self->to_bytes();
-
 	# wif network - 1B
-	my $wifdata = $self->network->wif_byte;
-
+	my $wifdata = $self->network->{wif_byte};
 	# key entropy - 32B
 	$wifdata .= ensure_length $bytes, $config{key_max_length};
-
 	# additional byte for compressed key - 1B
 	$wifdata .= $config{wif_compressed_byte} if $self->compressed;
 
@@ -59,11 +51,8 @@ sub from_wif
 	}
 
 	my $wif_network_byte = substr $decoded, 0, 1;
-	my @found_networks =
-		Bitcoin::Crypto::Network->find(sub { shift->wif_byte eq $wif_network_byte });
-	@found_networks = first { $_ eq $network }
-		@found_networks
-		if defined $network;
+	my @found_networks = find_network(wif_byte => $wif_network_byte);
+	@found_networks = first { $_ eq $network } @found_networks if defined $network;
 
 	Bitcoin::Crypto::Exception::KeyCreate->raise(
 		"found multiple networks possible for given WIF"
@@ -98,7 +87,7 @@ sub get_public_key
 __END__
 =head1 NAME
 
-Bitcoin::Crypto::Key::Private - Bitcoin private keys
+Bitcoin::Crypto::Key::Private - class for Bitcoin private keys
 
 =head1 SYNOPSIS
 
@@ -209,7 +198,7 @@ Returns current key instance.
 
 	sig: set_network($self, $val)
 
-Change key's network state to $val. It can be either network name present in Bitcoin::Crypto::Network package or an instance of this class.
+Change key's network state to $val. It can be either network name present in Bitcoin::Crypto::Network package or a valid network hashref. This will change the WIF generated by to_wif() method and also enable creation of public keys generating this network's addresses.
 
 Returns current key instance.
 
@@ -245,13 +234,11 @@ This module throws an instance of L<Bitcoin::Crypto::Exception> if it encounters
 
 =over 2
 
-=item * Sign - couldn't sign the message correctly
+=item KeySign - couldn't sign the message corretcly
 
-=item * Verify - couldn't verify the message correctly
+=item KeyCreate - key couldn't be created correctly
 
-=item * KeyCreate - key couldn't be created correctly
-
-=item * NetworkConfig - incomplete or corrupted network configuration
+=item NetworkConfig - incomplete or corrupted network configuration
 
 =back
 
