@@ -1,9 +1,6 @@
 package Bitcoin::Crypto::Key::Public;
 
-our $VERSION = "0.997_01";
-
-use v5.10;
-use warnings;
+use Modern::Perl "2010";
 use Moo;
 
 use Bitcoin::Crypto::Script;
@@ -12,8 +9,6 @@ use Bitcoin::Crypto::Bech32 qw(encode_segwit);
 use Bitcoin::Crypto::Config;
 use Bitcoin::Crypto::Helpers qw(hash160);
 
-use namespace::clean;
-
 with "Bitcoin::Crypto::Role::BasicKey";
 
 sub _is_private { 0 }
@@ -35,7 +30,7 @@ sub witness_program
 sub get_legacy_address
 {
 	my ($self) = @_;
-	my $pkh = $self->network->p2pkh_byte . $self->key_hash;
+	my $pkh = $self->network->{p2pkh_byte} . $self->key_hash;
 	return encode_base58check($pkh);
 }
 
@@ -56,9 +51,9 @@ sub get_segwit_address
 	# network field is not required, lazy check for completeness
 	Bitcoin::Crypto::Exception::NetworkConfig->raise(
 		"no segwit_hrp found in network configuration"
-	) unless defined $self->network->segwit_hrp;
+	) unless defined $self->network->{segwit_hrp};
 
-	return encode_segwit($self->network->segwit_hrp, $self->witness_program);
+	return encode_segwit($self->network->{segwit_hrp}, $self->witness_program);
 }
 
 1;
@@ -66,7 +61,7 @@ sub get_segwit_address
 __END__
 =head1 NAME
 
-Bitcoin::Crypto::Key::Public - Bitcoin public keys
+Bitcoin::Crypto::Key::Public - class for Bitcoin public keys
 
 =head1 SYNOPSIS
 
@@ -146,7 +141,7 @@ Returns current key instance.
 
 	sig: set_network($self, $val)
 
-Change key's network state to $val. It can be either network name present in Bitcoin::Crypto::Network package or an instance of this class.
+Change key's network state to $val. It can be either network name present in Bitcoin::Crypto::Network package or a valid network hashref. This will change the address.
 
 Returns current key instance.
 
@@ -183,11 +178,9 @@ This module throws an instance of L<Bitcoin::Crypto::Exception> if it encounters
 
 =over 2
 
-=item * KeyCreate - key couldn't be created correctly
+=item KeyCreate - key couldn't be created correctly
 
-=item * Verify - couldn't verify the message correctly
-
-=item * NetworkConfig - incomplete or corrupted network configuration
+=item NetworkConfig - incomplete or corrupted network configuration
 
 =back
 
