@@ -1,49 +1,42 @@
 package Bitcoin::Crypto::Role::ExtendedKey;
 
-our $VERSION = "0.997_01";
-
-use v5.10;
-use warnings;
+use Modern::Perl "2010";
+use Moo::Role;
 use List::Util qw(first);
-use Types::Standard qw(Str);
 
 use Bitcoin::Crypto::Key::Private;
 use Bitcoin::Crypto::Key::Public;
 use Bitcoin::Crypto::Config;
-use Bitcoin::Crypto::Types qw(IntMaxBits);
+use Bitcoin::Crypto::Types qw(IntMaxBits StrExactLength);
 use Bitcoin::Crypto::Util qw(get_path_info);
-use Bitcoin::Crypto::Helpers qw(pad_hex ensure_length hash160 verify_bytestring);
-use Bitcoin::Crypto::Network;
+use Bitcoin::Crypto::Helpers qw(pad_hex ensure_length hash160);
+use Bitcoin::Crypto::Network qw(find_network get_network);
 use Bitcoin::Crypto::Base58 qw(encode_base58check decode_base58check);
 use Bitcoin::Crypto::Exception;
-use Moo::Role;
 
 with "Bitcoin::Crypto::Role::Key";
 
 has "depth" => (
 	is => "ro",
-	isa => IntMaxBits [8],
-	coerce => 1,
+	isa => IntMaxBits[8],
 	default => 0
 );
 
 has "parent_fingerprint" => (
 	is => "ro",
-	isa => Str->where(q{ length $_ == 4 }),
+	isa => StrExactLength[4],
 	default => sub { pack "x4" }
 );
 
 has "child_number" => (
 	is => "ro",
-	isa => IntMaxBits [32],
-	coerce => 1,
+	isa => IntMaxBits[32],
 	default => 0
 );
 
 has "chain_code" => (
 	is => "ro",
-	isa => Str->where(q{ length $_ == 32 }),
-	required => 1,
+	isa => StrExactLength[32]
 );
 
 sub _build_args
@@ -73,29 +66,24 @@ sub to_serialized
 {
 	my ($self) = @_;
 
-	my $version =
-		$self->_is_private ? $self->network->extprv_version : $self->network->extpub_version;
+	my $network_key = "ext" . ($self->_is_private ? "prv" : "pub") . "_version";
+	my $version = $self->network->{$network_key};
 
 	# network field is not required, lazy check for completeness
 	Bitcoin::Crypto::Exception::NetworkConfig->raise(
-		"no extended key version found in network configuration"
+		"no $network_key found in network configuration"
 	) unless defined $version;
 
 	# version number (4B)
 	my $serialized = ensure_length pack("N", $version), 4;
-
 	# depth (1B)
 	$serialized .= ensure_length pack("C", $self->depth), 1;
-
 	# parent's fingerprint (4B) - ensured
 	$serialized .= $self->parent_fingerprint;
-
 	# child number (4B)
 	$serialized .= ensure_length pack("N", $self->child_number), 4;
-
 	# chain code (32B) - ensured
 	$serialized .= $self->chain_code;
-
 	# key entropy (1 + 32B or 33B)
 	$serialized .= ensure_length $self->raw_key, $config{key_max_length} + 1;
 
@@ -105,13 +93,10 @@ sub to_serialized
 sub from_serialized
 {
 	my ($class, $serialized, $network) = @_;
-	verify_bytestring($serialized);
-
 	# expected length is 78
 	if (defined $serialized && length $serialized == 78) {
 		my $format = "a4aa4a4a32a33";
-		my ($version, $depth, $fingerprint, $number, $chain_code, $data) =
-			unpack($format, $serialized);
+		my ($version, $depth, $fingerprint, $number, $chain_code, $data) = unpack($format, $serialized);
 
 		my $is_private = pack("x") eq substr $data, 0, 1;
 
@@ -123,15 +108,9 @@ sub from_serialized
 			if $is_private;
 
 		$version = unpack "N", $version;
-		my @found_networks = Bitcoin::Crypto::Network->find(
-			sub {
-				my ($inst) = @_;
-				return ($class->_is_private ? $inst->extprv_version : $inst->extpub_version) eq
-					$version;
-			}
-		);
-		@found_networks = first { $_ eq $network }
-			@found_networks if defined $network;
+		my $network_key = "ext" . ($class->_is_private ? "prv" : "pub") . "_version";
+		my @found_networks = find_network($network_key => $version);
+		@found_networks = first { $_ eq $network } @found_networks if defined $network;
 
 		Bitcoin::Crypto::Exception::KeyCreate->raise(
 			"found multiple networks possible for given serialized key"
@@ -155,8 +134,7 @@ sub from_serialized
 		$key->set_network(@found_networks);
 
 		return $key;
-	}
-	else {
+	} else {
 		Bitcoin::Crypto::Exception::KeyCreate->raise(
 			"input data does not look like a valid serialized extended key"
 		);
@@ -179,8 +157,9 @@ sub from_serialized_base58
 sub get_basic_key
 {
 	my ($self) = @_;
+	my $entropy = $self->raw_key;
 	my $base_class = "Bitcoin::Crypto::Key::" . ($self->_is_private ? "Private" : "Public");
-	my $basic_key = $base_class->new($self->key_instance);
+	my $basic_key =  $base_class->from_bytes($entropy);
 	$basic_key->set_network($self->network);
 
 	return $basic_key;
@@ -212,7 +191,6 @@ sub derive_key
 	my $key = $self;
 	for my $child_num (@{$path_info->{path}}) {
 		my $hardened = $child_num >= $config{max_child_keys};
-
 		# dies if hardened-from-public requested
 		# dies if key is invalid
 		$key = $key->_derive_key_partial($child_num, $hardened);
